/*
 *	@(#) page.h 1.3 86/12/17 
 *
 *	Copyright (C) The Santa Cruz Operation, 1984, 1985, 1986, 1987.
 *	Copyright (C) Microsoft Corporation, 1984, 1985, 1986, 1987.
 *	This Module contains Proprietary Information of
 *	The Santa Cruz Operation, Microsoft Corporation
 *	and AT&T, and should be treated as Confidential.
 */

#ifdef M_I386

/*
 * Format of a page directory or page table entry
 *
 *	N.B.: if te_pres is zero, the other bits have no assigned meaning,
 *	      and are all available for software use.
 *
 *	When the page table entry is marked present, the page frame number
 *	(te_frameno) refers to a memory frame.  When the PTE is marked 
 *	not present, te_frameno refers to a swap frame.
 */
struct tabent {
	unsigned long	te_flags:9;	/* see below */
	unsigned long	te_type:3;	/* see below */
	unsigned long	te_frameno:20;	/* see below */
};

/*
 * values of te_flags
 */
#define TF_PRESENT	0x001	/* page is present */
#define TF_WRITE	0x002	/* page is writable */
#define TF_USER		0x004	/* access is kernel only */
#define TF_RES1		0x008	/* reserved must not be set */
#define TF_RES2		0x010	/* reserved must not be set */
#define TF_ACCESSED	0x020	/* page has been accessed */
#define TF_DIRTY	0x040	/* page has been written */
#define TF_RES3		0x080	/* reserved must not be set */
#define TF_RES4		0x100	/* reserved must not be set */

/*
 * values of te_type
 */
#define TE_INVALID	0	/* page is not valid */
#define TE_FILSYS	1	/* text page resides on filesystem */
#define TE_SWAP		3	/* page resides on swap area */
#define TE_ZERO		4	/* zero fill on access */
#define TE_COPY		5	/* copy on write */
#define TE_INTRN	6	/* filesystem page in transit */

/* 
 * format of a linear address
 */
struct laddr {
	unsigned la_off:12;		/* offset of byte in page */
	unsigned la_page:10;		/* offset of pte in page table */ 
	unsigned la_dir:10;		/* offset of pde in page directory */
};

/*
 * layout of a memory page frame information structure 
 */
struct mpfi {
	struct mpfi *mp_fw, *mp_bw;	/* freelist chains */
	struct dpfi *mp_xlink;		/* link to other map */
	struct mpfi *mp_hash;		/* link for filsys cache list */
	unsigned short mp_rcnt;		/* reference count */
	unsigned short mp_flags;	/* see below for values */
	dev_t	mp_dev;			/* device for filsys page */
	ino_t	mp_ino;			/* inode for filsys page */
	daddr_t mp_blockno;		/* block for filsys page */
};
extern struct mpfi mpfi[];

/*
 * values of mp_flags
 */
#define MF_INTRANSIT	1
#define MF_NOTIFY	2
#define MF_DIRTY	4
#define MF_CACHE	8

struct dpfi {
	struct dpfi *dp_fw, *dp_bw;	/* freelist chains */
	struct mpfi *dp_xlink;
	unsigned short dp_flags;
	unsigned short dp_rcnt;
};
extern struct dpfi dpfi[];

/*
 * values of dp_flags
 */
#define DF_INTRANSIT	1
#define DF_NOTIFY	2
#define DF_DIRTY	4


/*
 * values of cr0 register
 */
#define CR0_PE	0x00000001	/* protection enabled (segmentation) */
#define CR0_MP	0x00000002	/* monitor coprocessor */
#define CR0_EM	0x00000004	/* emulate coprocessor */
#define CR0_TS	0x00000008	/* task switched */
#define CR0_ET	0x00000010	/* coprocessor type */
#define CR0_PG	0x80000000	/* paging enabled */

/*
 * arguments to chgprot
 */
#define RO	0
#define RW	TF_WRITE

/*
 * some useful constants
 */
#define PGSIZE	ctob(1)
#define NEPT	(ctob(1)/sizeof(struct tabent))
#define NBPT	(NEPT*ctob(1))

/* how many pde's needed to map n bytes */
#define btopde(n) (((n) + NBPT - 1)/NBPT)
#define btopdet(n) ((n)/NBPT)

/* how many page table entries to map n bytes */
#define btopte(n) (((n) + ctob(1) - 1)/ctob(1))
#define btoptet(n) ((n)/ctob(1))

/* virtual address to page offset */
#define	vtpo(a)		((unsigned long)(a) & (PGSIZE-1))

/* virtual address to page number */
#define	vtpn(a)		( btoct((a)) )

#endif
